; Copyright (c) 1991-1992, John David Rohner.  All rights reserved.

;
; This routine works similar to Basic's CHR$.  It's actually a direct
; substitute.  But Basic's BC piled on a lot of extra coding after each call,
; both slowing it down and increasing the program size significantly (it
; does an unnecessary $SASS call after each call).  So until I resolve this
; problem, this routine will just serve as an example.
;
; Usage:  Result$ = AscChr(Input%)
; Declare:  DECLARE FUNCTION AscChr$ (BYVAL Input%)
;
; Only the LSB of the Input% is used, so you can send larger numbers than
; 255.  The MSB will be ignored (so if you do this, better know what'll be
; in your lowest 8 bits).
;

.Model Medium, Basic
.Code


; DGROUP allows us to use Basic's DGROUP area.
;
; Our buffer is a single string:
;   the first two bytes are the strings length (always one here),
;   then next two bytes are the actual string data's address
;   (the string 'AscChrBuf' has an address, but it includes all three: the
;   size, the data's address, and the data itself).
; Here, the last byte is our output/string/data buffer.
; Just like:
;   TYPE AnyString
;     Length  AS INTEGER
;     Address AS INTEGER
;     Data    AS STRING * Length
;   END TYPE
; Guess this is so the data can be anywhere--not contiguous to the descriptor
; information.
;
; So, when calling a routine that has string parameters:
;  An integer (called the string descriptor) address is put on the stack.
;  This address points to the string information integers.
;  The two string information integers are it's length, and the address of
;  the strings data.
;  mov bx,[bp + n]    gets the string descriptor integer address.
;  mov cx,[bx]        gets the string data's size (retrive the contents of
;                     the address pointed to by the string descriptor integer).
;  mov dx,[bx + 2]    gets the address of the string data, which is two bytes
;                     farther than the address pointed to by the string
;                     descriptor integer (thus Length and data address are
;                     right next to each other--making up four bytes, which
;                     is called the string descriptor information).
;

DGROUP      group   _DATA

_DATA       segment word public 'DATA'

AscChrBuf   DB      1,0,0,0,0

_DATA       ends

;
; Actual program code begins.
;

            PUBLIC  AscChr            ;Make this routine available to LINK.

AscChr      proc    far               ;Beginning of routine.

            push    bp
            mov     bp,sp
            mov     cx,[bp + 06]      ;Get number of ASCII code to find.
                                      ;All we really want is CL.
            lea     bx,AscChrBuf      ;Get the address of AscChrBuf.
            mov     ax,bx             ;We want to exit with this address,
                                      ;so put it in AX.
            add     bx,04             ;Add 4, so it points to the data.
            mov     [bx - 02],bx      ;Store the data pointer with the string
                                      ;information
            mov     [bx],cl           ;Actually store the output character.
            pop     bp
            ret     2                 ;Pop-return past our input parameter.


AscChr      endp                      ;End of routine.

End

