' $INCLUDE: 'BBS-VAR.BAS'
'
' Copyright (c) 1991-1992, John David Rohner.  All rights reserved.
'
'Various internal routines:
'  DLTime
'  FAccessAllowed
'  MappedSL
'  MAccessAllowed
'  NextArea



        '* * * * * *
        ' Calculate the download/door minutes user has left today.
        '
        ' Date last checked for perfection: Nov 6 1992
        '
FUNCTION DLTime%
  DLTime% = Levels(UserSL).MinsPerDay + User.MinCredits - User.Elapsed
END FUNCTION
        '
        '* * * *




        '* * * * * *
        ' This routine returns the next or previous message or file
        ' area that the current user can access.
        '
        ' p  What to do:
        '     1 to go to next file area
        '    -1 to go to previous file area
        '     2 to go to next message area
        '    -2 to go to previous message area
        '
        ' FileIndex is relied upon and returns the next file area, or
        ' doesn't change.
        '
        ' BaseIndex is relied upon and returns the next message area,
        ' or doesn't change.
        '
        ' 32767 goes to first area or last area depending on selection.
        '
        ' SL level is based on Scan level for the area.
        '
        ' It's up to the calling routine to match what was sent to
        ' determine if at a quit/end/start point.
        '
        ' Date last checked for perfection: Nov 6 1992
        '
SUB NextArea (p)

  SELECT CASE p
    CASE 1
         SELECT CASE FileIndex
           CASE 0, 32767
                FOR K0 = 1 TO DirsSize
                  IF FAccessAllowed(K0) THEN FileIndex = K0 : _
                                             EXIT SUB
                NEXT
           CASE ELSE
                K0 = FileIndex
                DO
                  FileIndex = FileIndex + 1
                  IF FileIndex > DirsSize THEN FileIndex = 1
                LOOP UNTIL FAccessAllowed(FileIndex) OR FileIndex = K0
         END SELECT
    CASE -1
         SELECT CASE FileIndex
           CASE 0. 32767
                FOR K0 = DirsSize TO 1 STEP -1
                  IF FAccessAllowed(K0) THEN FileIndex = K0 : _
                                             EXIT FOR
                NEXT
           CASE ELSE
                K0 = FileIndex
                DO
                  FileIndex = FileIndex - 1
                  IF FileIndex = 0 THEN FileIndex = DirsSize
                LOOP UNTIL FAccessAllowed(FileIndex) OR FileIndex = K0
         END SELECT
    CASE 2
         SELECT CASE BaseIndex
           CASE 32767
                FOR K0 = 0 TO BasesSize
                  IF MAccessAllowed(K0) THEN BaseIndex = K0 : _
                                             EXIT FOR
                NEXT
           CASE ELSE
                K0 = BaseIndex
                DO
                  BaseIndex = BaseIndex + 1
                  IF BaseIndex > BasesSize THEN BaseIndex = 0
                LOOP UNTIL MAccessAllowed(BaseIndex) OR BaseIndex = K0
         END SELECT
    CASE -2
         SELECT CASE BaseIndex
           CASE 32767
                FOR K0 = BasesSize TO 0 STEP -1
                  IF MAccessAllowed(K0) THEN BaseIndex = K0 : _
                                             EXIT FOR
                NEXT
           CASE ELSE
                K0 = BaseIndex
                DO
                  BaseIndex = BaseIndex - 1
                  IF BaseIndex = -1 THEN BaseIndex = BasesSize
                LOOP UNTIL MAccessAllowed(BaseIndex) OR BaseIndex = K0
         END SELECT
  END SELECT

END SUB
        '
        '* * * *




        '* * * * * *
        ' Check to see if the user has access to a file area.
        '
        ' p  file area to check
        '
        ' returns with 0 if no access allowed, or -1 if they can
        ' access the area.
        '
        ' Access is based on the file area's ScanSL, or whether
        ' they are undergoing Peer Review.
        '
        ' Date last checked for perfection: Sep 21 1992
        '
FUNCTION FAccessAllowed% (p)

  '
  ' The user must have an SL at least that of the file area's ScanSL.
  ' If the user is undergoing Peer Review, then the area must have
  ' the PRL flag set for them to have access.
  '
  IF (Levels(UserSL).SecLevel < FileBases(p).ScanSL) _
     OR (BitTest(User.Attr,10) AND NOT BitTest(FileBases(p).Attr,7)) _
     THEN FAccessAllowed% = 0 _
     ELSE FAccessAllowed% = -1

END FUNCTION
        '
        '* * * *




        '* * * * * *
        ' Check to see if the user has access to a message area.
        '
        ' p  message area to check
        '
        ' returns with 0 if no access allowed, or -1 if they can
        ' access the area.
        '
        ' Access is based on the message area's ScanSL, or whether
        ' they have it toggled OFF, or are locked out.
        '
        ' Date last checked for perfection: Sep 21 1992
        '
FUNCTION MAccessAllowed% (p)

  '
  ' The user must have an SL at least that of the message area's ScanSL.
  ' Areas that the user is locked out of or has turned off are considered
  ' inaccessible.
  '
  IF (Levels(UserSL).SecLevel < MsgBases(p).ScanSL) _
     OR (CVL(MID$(UserMsgInfo$,p * 4 + 1,4)) < 0) _
     THEN MAccessAllowed% = 0 _
     ELSE MAccessAllowed% = -1

END FUNCTION
        '
        '* * * *





        '* * * * * *
        ' This routine finds the SL given the SL value (eg. given 5 it
        ' returns 1, because the SL of one has a defined value of 5 to
        ' 9 (2 = 10 to 19, etc.) as defined in SETUP.INI.
        '
        ' p  is the SL value to match.
        '
        ' returns with the SL that the SL value matches.
        '
        ' Date last checked for perfection: Sep 21 1992
        '
FUNCTION MappedSL% (p)

  FOR K0 = LevelsSize TO 2 STEP -1
   IF p >= Levels(K0).SecLevel THEN EXIT FOR
  NEXT
  MappedSL = K0

END FUNCTION
        '
        '* * * *

