; Copyright (c) 1991-1992, John David Rohner.  All rights reserved.

;
; This routine uncompresses a 5-bit based string into an ASCII 8-bit string.
; 
; Usage:  Result$ = Make8Bit$ (5BitString$, RecNum%, Size%)
; Declare:  DECLARE FUNCTION Make8Bit$ (5BitString$, BYVAL RecNum%, BYVAL Size%)
;
; Where 5BitString$ is the string to uncompress from 5 bits to 8 bits, RecNum%
; is the record number the word was given when stored, Size% is the size of the
; desired output string, and Result$ is the 8 bit output string of size Size%.
;
; The record number is necessary because we need to calculate the "bit start"
; position within the string we're given.
;

.Model Medium, Basic
.Code

DGROUP      group   _DATA

_DATA       segment word public 'DATA'

Max18Buf    DB      22 dup (0)

_DATA       ends


            PUBLIC  Make8Bit          ;Make this routine available to LINK.

Make8Bit    proc    far               ;Beginning of routine.

            push    bp
            mov     bp,sp

            mov     bx,[bp + 0AH]     ;Get descriptor of the string.
            mov     cx,[bx]           ;Move size of string into CX.
            jcxz    Done              ;If a null string, then exit.
            mov     si,[bx + 02]      ;String location -> SI.

            mov     cx,[bp + 06]      ;Get size of desired output string.
            push    cx                ;Save size for later.
            mov     ax,cx             ;Put size of word in ax.
            mov     bx,5              ;Size of word * 5 bits = size in bits.
            mul     bx                ;(AX * 5 -> DX:AX)
            mov     bx,[bp + 08]      ;Get record number of this word.
            dec     bx                ;0 based.
            mul     bx                ;Get bit pos in terms of 5 bits.

            and     ax,0007H          ;Mask off all but first 3 bits.
            mov     dx,ax             ;The remainder, put in DX.

            lea     di,Max18Buf       ;Point to our output string.
            add     di,04             ;Point to the data area of that string.

            lodsb                     ;Get the first two bytes.
            mov     ah,al             ;Putting first letter in AH,
            lodsb                     ;and second in AL.
            mov     bx,ax             ;Save for next time.

        R1: mov     ax,bx             ;Original AX back.
            cmp     dx,12             ;Are we out of 5 bit words?
            jl      R2                ;Nope, get next.
            mov     ah,al             ;Yep, shift AL to AH,
            lodsb                     ;and get another letter into AL.
            mov     bx,ax             ;Save our new AX.
            sub     dx,8              ;Back up some bits.

        R2: push    cx                ;Save our size.
            mov     cl,dl             ;Move starting bit position into CL.
            shl     ax,cl             ;Shift CL over those bits,
            mov     cl,3              ;Shift it back 3,
            shr     ax,cl             ;now AH has our uncompressed letter.
            pop     cx                ;Get the size back.
            add     ah,64             ;Adjust to real-world ASCII.
            cmp     ah,64             ;A 64 means we had a 0--which is a space.
            jne     R3                ;No space, so jump.
            mov     ah,32             ;Otherwise replace it with a space.

        R3: mov     [di],ah           ;Store it into our output string.
            inc     di                ;Move to next position.
            add     dx,5              ;Shift over the 5 bit word in DX.
            loop    R1

      Done: lea     di,Max18Buf       ;Find our output string.
            pop     cx                ;Get our desired length back.
            push    di                ;Save the output string's address.
            mov     [di],cx           ;Give it the proper size.
            add     di,04             ;Point to the data area.
            mov     [di - 02],di      ;Put address of data area in.
            pop     ax                ;We exit with the address.

            pop     bp
            ret     6                 ;Pop-return past our input parameters.

Make8Bit    endp                      ;End of routine.

End


