;
; This routine will write a data string to a file at a specific location.
;
; Usage:  CALL FilePutSLoc(Handle%, Location&, Input$)
; Declare: DECLARE SUB FilePutSLoc (BYVAL Handle%, BYVAL Location&, Input$)
;
; Where Handle% is the file handle to use, Location& is the location - 1 to
; write to, and Input$ contains the data to write out.
;
; We only write out for the length of Input$.
;
; Location& specifies the location to write the data to--using a 0..n scale,
; which means we actually write to location Location& + 1.
;
; It does no negative number checking.  A zero put's it at position 1.  A 3
; put's it on the 4th character, and so on.  Thus, no more need for the "+ 1"
; when doing: (rec - 1) * len + 1   to position it, now (rec - 1) * len does
; it.
;
; Because null strings are ignored, this routine cannot be used to truncate a
; file.  To do that, use FilePutLoc with a size field of zero.
;

.Model Medium, Basic
.Code

            PUBLIC  FilePutSLoc       ;Make this routine available to LINK.

FilePutSLoc proc    far               ;Beginning of routine.

            push    bp
            mov     bp,sp

            mov     si,[bp + 06]      ;Descriptor of Input$.
            mov     cx,[si]           ;
            jcxz    Done              ;Quick exit if null string.

            push    cx                ;Save the length for later.
            mov     bx,[bp + 0CH]     ;Get the file handle into BX.
            mov     ax,4200H          ;Subfunction to seek to loc in file.
            mov     dx,[bp + 08]      ;Low order word of long offset.
            mov     cx,[bp + 0AH]     ;High order word of long offset.
            int     21H               ;Call DOS Services to do it.

            mov     ah,40H            ;Subfunction to write to a file.
            pop     cx                ;Length to write out.
            mov     dx,[si + 02]      ;Data to write out.
            int     21H               ;CALL DOS Services to do it.

      Done: pop     bp
            ret     8                 ;Pop-return past our input parameters.

FilePutSLoc endp                      ;End of routine.

End

