; Copyright (c) 1991-1992, John David Rohner.  All rights reserved.

;
; This routine will return a string containing a word within the input string
; referenced to by an input value.  It returns word number x from a string.
;
; Usage:  Result$ = WordsGet$(Input$, FindNum%, Start%)
; Declare: DECLARE FUNCTION WordsGet$ (Input$, BYVAL FindNum%, Start%)
;
; Where Input$ is the string to search, FindNum% is the word number to get,
; Result$ is the gotten word, and Start% is the position in the string where
; the word starts.
;
; A null string, or a FindNum% that is larger than the number of words in the
; line will result in a null output string, and Start% could contain anything.
; That is, check for a null string, if you don't have one, everything's fine.
; But if you do have a null string, then don't use Start%.
;
; Words greater than 80 characters are truncated to 80 characters.
;


.Model Medium, Basic
.Code


DGROUP      group   _DATA

_DATA       segment word public 'DATA'

WordsGetBuf DB      84 dup (0)

_DATA       ends

;
; Actual program code begins.
;

            PUBLIC  WordsGet          ;Make this routine available to LINK.

WordsGet    proc    far               ;Beginning of routine.

            push    bp
            mov     bp,sp
            mov     bx,[bp + 0AH]     ;Get Input$'s desciptor -> BX.
            mov     cx,[bx]           ;Get it's length.
            jcxz    D1                ;If a null string, then exit.
            mov     si,[bx + 02]      ;Get Input$'s addresses -> SI.
            mov     di,[bp + 08]      ;Get the number of the word to find.
            xor     ax,ax             ;Our 'in a word?' flag.
            mov     bx,ax             ;Start at position zero.
            dec     ax                ;We start out 'not in a word'.
        R1: cmp byte ptr [bx + si],20H   ;Is the character a space?
            je      R2                ;Yes, get next character.
            or      ax,ax             ;Are we in a word now?
            je      R3                ;If not in a word, then get next.
            dec     di                ;Reduce our words-to-find count.
            or      di,di             ;If we're at the one we want,
            je      R4                ;then go finish up.
            xor     ax,ax             ;Now, we're 'not in a word'.
            jmp     R3

        R2: dec     ax                ;Put us, 'in a word'.

        R3: inc     bx                ;Move to next character.
            loop    R1                ;Keep looping for all of Input$.
            jmp     D1                ;No more found, exit.

        R4: mov     dx,bx             ;Save bx for later.
            inc     bx                ;Clean up, make it a real-world value.
            mov     di,[bp + 06]      ;Store our starting position,
            mov     [di],bx           ;in Start%.
            xor     ax,ax             ;AX will contain our length.
        R5: cmp byte ptr [bx + si],20H   ;Looking for a space.
            je      R6                ;Found, finish up.
            inc     bx                ;Try next character.
            inc     ax                ;Add to our length total.
            loop    R5                ;Keep looping until done.
            dec     ax                ;No last space, clean up.
        R6: inc     ax                ;More cleaning up.
            xchg    cx,ax             ;We want the length in CX.

        D1: lea     di,WordsGetBuf    ;Find our output buffer.
            mov     [di],cx           ;Put it's length in.
            push    di                ;Save it for AX.
            add     di,04             ;Point to data area.
            mov     [di - 02],di      ;Put address of data area in.
            jcxz    Done              ;Nothing to do, so exit.
            mov     bx,dx             ;Get back our Start%.
            cmp     cx,81             ;If bigger than our buffer,
            jb      D2
            mov     cx,80             ;then truncate it.
        D2: mov     al,[bx + si]      ;Get a character from Input$,
            mov     [di],al           ;and put in in the output buffer.
            inc     di
            inc     bx
            loop    D2                ;Do it for the length of CX.

      Done: pop     ax                ;AX exits with our string.
            mov     sp,bp
            pop     bp
            ret     6                 ;Pop-return past our input parameters.

WordsGet    endp                      ;End of routine.

End

